/* PASM demo for use with Catalina IDE Precompiler and Compiler */

#include <stdio.h>

/* Global constants for both C and PASM */
const unsigned long int graphicstart  = 0;                           // start of space for video graphics buffer
const unsigned long int graphicsize   = 19200;                       // size of graphics buffer = 19200 bytes
const unsigned long int keyboardstart = 19200;                       // start of space for keyboard
const unsigned long int keyboardsize  = 100;                         // size of space for keyboard
/* End global constants */

/* PASM Code for compilation using SpinC and inclusion as a data file
PASM Start mycogject.spin
CON
                                                                     ' add your values here
  _clkfreq = 80_000_000                                              ' 5Mhz Crystal
  _clkmode = xtal1 + pll16x                                          ' x 16
  graphicstart = 2048                                                ' precompiler will overwrite this

PUB Main
    coginit(1,@cogstart,0)                                           ' cog 1, cogstart, dummy value

DAT
                  org 0
cogstart          wrbyte  testvalue, testvariable                    ' test value A
                  jmp #cogstart

testvariable      long    graphicstart                               ' test memory location at graphicstart
testvalue         long    65                                         ' ascii A
                  fit     496
PASM End
*/ 

void mycogject(int cognumber)                                        // this name copied from the .spin name in the pasm section - names must match eg void mycogject matches mycogject.spin
{
       unsigned long mycogject_array[] =                               // dummy data or compiled above code or #include file.h
       {
              0x003c0602, 0x5c7c0000, 0x00001388, 0x00000041
       };
       unsigned long par_cogject[] = { 1, 2, 3 };                    // data to pass to cog - ignore if not used
       _coginit((int)par_cogject>>2, (int)mycogject_array>>2, cognumber);  // array name built from spin file name
}

void clearscreen()                                                   // white text on dark blue background
{
       int i;
       for (i=0;i<40;i++)
       {
               t_setpos(0,0,i);                                      // move cursor to next line
               t_color(0,0x08FC);                                    // RRGGBBxx eg dark blue background 00001000 white text 11111100
       }
}

void sleep(int milliseconds)                                         // sleep function
{
       _waitcnt(_cnt()+(milliseconds*(_clockfreq()/1000))-4296);
}

char peek(int address)                                               // function implementation of peek
{
       return *((char *)address);
}

void poke(int address, char value)                                   // function implementation of poke
{
       *((char *)address) = value;
}

void main ()
{
       char c;
       int i;
       clearscreen();
       printf("Clock speed %u \n",_clockfreq());                     // see page 28 of the propeller manual for other useful commands
       printf("Catalina running in cog number %i \n",_cogid());      // integer
       i=graphicstart;                                               // location to peek and poke
       poke(i,66);                                                   // poke a B, the cog should change this to an A
       printf("load cogject into cog 7\n");
       mycogject(7);                                                 // run in cog 7
       c=peek(i);                                                    // peek byte at this location
       printf("Peek value at %i = character %c Ascii value %d \n",i,c,c);
       while (1); // Prop reboots on exit from main()
}

